package com.qmth.teachcloud.common.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.dto.BlurryUserDto;
import com.qmth.teachcloud.common.bean.dto.LoginDto;
import com.qmth.teachcloud.common.bean.dto.UserDto;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.bean.params.UserSaveParams;
import com.qmth.teachcloud.common.bean.params.UserSaveReportParams;
import com.qmth.teachcloud.common.bean.result.LoginResult;
import com.qmth.teachcloud.common.bean.result.SysUserResult;
import com.qmth.teachcloud.common.bean.result.UserRoleNameResult;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.FlowApproveNameEnum;
import com.qmth.teachcloud.common.enums.RoleTypeEnum;
import com.qmth.teachcloud.common.enums.userPush.SpecialPrivilegeEnum;
import org.springframework.util.LinkedMultiValueMap;

import javax.servlet.http.HttpServletResponse;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 用户表 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface SysUserService extends IService<SysUser> {


    /**
     * 知学知考系统 - 查询用户列表
     *
     * @param userInfo   用户信息（账号/姓名/手机号）
     * @param orgId      机构id
     * @param roleId     角色id
     * @param enable     启用禁用状态
     * @param pageNumber 分页页数
     * @param pageSize   分页容量
     * @return 用户集合
     */
    IPage<UserDto> listForPrintSystem(String userInfo, Long orgId, String roleId, Boolean enable, Integer pageNumber, Integer pageSize);

    /**
     * 教研分析系统 - 查询用户列表
     *
     * @param roleId     角色id
     * @param enable     启用禁用状态
     * @param loginName  账号
     * @param realName   真实姓名
     * @param pageNumber 分页页数
     * @param pageSize   分页容量
     * @return 用户集合
     */
    IPage<UserDto> listForReportSystem(String roleId, Boolean enable, String loginName, String realName, Integer pageNumber, Integer pageSize);

    Long saveUser(UserSaveParams userSaveParams) throws IllegalAccessException;

    /**
     * 保存用户
     *
     * @param userSaveReportParams
     * @return
     */
    boolean saveUserReport(UserSaveReportParams userSaveReportParams);

    boolean enable(SysUser user) throws NoSuchAlgorithmException, IllegalAccessException;

    boolean enableReport(SysUser user) throws NoSuchAlgorithmException;

    boolean resetPassword(Long id) throws NoSuchAlgorithmException;

    /**
     * 更新用户登录信息（密码、手机号）
     * <p>
     * 1.判断用户在系统中是否存在
     * 2.判断用户原密码是否正确
     * 3.更新密码 -> 新旧密码不一致判断
     * 4.更新手机 -> 检验验证码正确新（不为空、正确、未过期）、手机号唯一性校验
     * 5.若用户已经登陆且进行了密码或手机号修改 -> 清空用户登录信息
     * 6.如果不是共用验证码则过期该用户验证码
     * 7.调用用户登录公共方法（验证码参数使用系统共用验证码）登录用户
     * </p>
     *
     * @param user 用户信息
     * @return 登录信息
     * @throws NoSuchAlgorithmException 异常
     */
    LoginResult updatePassword(SysUser user) throws NoSuchAlgorithmException;

    boolean bindRoles(SysUser sysUser);

    LoginDto getUserRoles(SysUser sysUser);

    List<BlurryUserDto> listUser(String courseCode, String param);

    List<SysUser> listByOrgId(Long id);

    boolean saveCustomer(UserSaveParams userSaveParams);

    IPage<UserDto> listCustomer(String realName, Boolean enable, Integer pageNumber, Integer pageSize);

    /**
     * 根据外键查找教师信息
     *
     * @param schoolId    学校id
     * @param teacherCode 教师编号
     * @return 学院数据
     */
    SysUser findByForeignKey(Long schoolId, String teacherCode);

    /**
     * 临时-不鉴权保存用户 （默认创建用户角色为该学校管理员）
     *
     * @param userSaveParams 用户参数
     * @return 结果
     */
    boolean saveUserNoAuth(UserSaveParams userSaveParams);

    /**
     * 过滤拥有审核权限的人
     *
     * @param schoolId
     * @param orgIds
     * @param filter
     * @param approveName
     * @param realName
     * @return
     */
    List<ApproveUserResult> filterFlowPrivilege(Long schoolId, Set<Long> orgIds, Boolean filter, FlowApproveNameEnum approveName, String realName);

    /**
     * 执行批量导入用户逻辑
     *
     * @param finalList 基础课程数据集合
     * @param map       参数
     * @return 结果
     */
    Map<String, Object> executeSysUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws NoSuchAlgorithmException;

    /**
     * 根据id获取
     *
     * @param id
     * @return
     */
    public List<ApproveUserResult> findById(Long id);

    /**
     * 根据id集合获取
     *
     * @param ids
     * @return
     */
    public ApproveUserResult findByIds(List<Long> ids);

    /**
     * 校验短信验证码
     *
     * @param userId       用户id
     * @param mobileNumber 用户手机号
     * @param code         验证码
     */
    void checkSmsCode(Long userId, String mobileNumber, String code);

    /**
     * 强行过期用户手机号验证码（过期时间改为验证码发送时间）
     *
     * @param userId       用户id
     * @param mobileNumber 用户手机号
     */
    void expiredVerifyCode(Long userId, String mobileNumber);

    /**
     * 获取教研室id底下的人数量
     *
     * @param orgId
     * @return
     */
    public int countByTeachingRoomId(Long orgId);

    /**
     * 根据用户id查询当前用户的特殊权限
     *
     * @param userId   用户id
     * @param schoolId 学校id
     * @return 用户特殊权限身份
     */
    SpecialPrivilegeEnum findUserSpecialPrivilegeByUserId(Long userId, Long schoolId);

    /**
     * 查找用户角色名
     *
     * @param userIds
     * @return
     */
    List<UserRoleNameResult> selectRoleNames(List<Long> userIds);

    /**
     * 根据id集合获取
     *
     * @param ids
     * @return
     */
    public List<ApproveUserResult> findByIdsForResult(List<Long> ids);

    /**
     * 根据用户id查找该用户所属的机构集合
     *
     * @param userId 用户id
     * @return 机构集合
     */
    List<Long> findOrgIdListByUserId(Long userId);

    /**
     * 用户是管理员数据权限？ 没有schoolId的用户和机构中包含 orgType为‘SCHOOL’
     *
     * @param userId 用户id
     * @return true：是管理员权限、false：不是
     */
    boolean isAdminDataAuth(Long userId);

    /**
     * 查找机构树下所有用户集合
     *
     * @return 用户集合
     */
    List<SysUserResult> findSysUserResultList();

    /**
     * 根据roleId和成员姓名获取
     *
     * @param roleId
     * @param name
     * @return
     */
    public List<ApproveUserResult> findByRoldIdAndMemberName(Long roleId, String name);

    List<SysUser> findBySchoolIdAndCode(Long schoolId, String code);

    /**
     * 查询某用户是否包含系统特定角色
     *
     * @param userId      用户id
     * @param defaultRole 系统默认角色
     * @return true:包含 || false:不包含
     */
    boolean includeSysRole(Long userId, RoleTypeEnum defaultRole);

    /**
     * 为没有学校管理员的学校自动创建一个学校管理员
     *
     * @param schoolId 学校id
     */
    void buildAdministratorForSchool(Long schoolId);

    boolean removeBasicCourseBatch(List<Long> idList);

    boolean remove(Long id) throws NoSuchAlgorithmException;

    void exportData(HttpServletResponse response, String userInfo, Long orgId, String roleId, Boolean enable) throws Exception;

    /**
     * 根据角色权限类型查询用户
     *
     * @param schoolId
     * @param specialPrivilegeEnum
     * @return
     */
    List<SysUserResult> findSysUserResultListNew(Long schoolId, SpecialPrivilegeEnum specialPrivilegeEnum);

    /**
     * 查找科目下命题老师
     *
     * @param schoolId
     * @param courseCode
     * @return
     */
    List<SysUserResult> findCourseSysUserResultList(Long schoolId, String courseCode);
}
